# Project Configuration: #

## To use the SDK, Link App Binary with following ##

### Frameworks ###
* AudioToolbox
* AVFoundation
* CFNetwork
* CoreAudio
* CoreGraphics
* CoreMedia
* Foundation
* GLKit
* UIKit
* VideoToolbox

### Dynamic Libraries ###

* libc.dylib
* libicucore.dylib
* libsqlite3.dylib
* libstdc++.dylib

## Build Settings ##

* Other Linker Flags = -ObjC

## External Dependencies ##

* Socket Rocket (https://github.com/square/SocketRocket)
* ios-ntp (https://github.com/jbenet/ios-ntp)

## Bridging Header File for Swift

```
#!Objective-C

#import <AudioToolbox/AudioToolbox.h>
#import <AVFoundation/AVFoundation.h>
#import <CFNetwork/CFNetwork.h>
#import <CoreGraphics/CoreGraphics.h>
#import <CoreMedia/CoreMedia.h>
#import <CoreVideo/CoreVideo.h>
#import <Foundation/Foundation.h>
#import <GLKit/GLKit.h>
#import <OpenGLES/EAGL.h>
#import <QuartzCore/QuartzCore.h>
#import <Security/Security.h>
#import <SystemConfiguration/SystemConfiguration.h>
#import <UIKit/UIKit.h>
#import <SKYLINK/SKYLINK.h>
```

# SDK Usage: #

Following is a sample usage of the SKYLINKConnection:

### import statement ###

```
#!Objective-C

#import<SKYLINK/SKYLINK.h>
```

### Connection configuration ###

```
#!Objective-C

SKYLINKConnectionConfig *config = [SKYLINKConnectionConfig new];
config.video = YES;
config.audio = YES;
config.fileTransfer = YES;
config.dataChannel = YES;
// The default timeout is 60 seconds. Any value less than or equal to zero is discarded.
config.timeout = 30;
// Only advanced users who know about the underlying webrtc should use the userInfo
// For e.g., to disable both TURN and STUN, and use tcp for transport:
config.userInfo = @{@"TURN": [NSNumber numberWithBool:TRUE], @"STUN": [NSNumber numberWithBool:TRUE], @"transport": @"tcp"}; // transport can be tcp/udp
```

### Connection delegates ###

```
#!Objective-C

SKYLINKConnection connection = [[SKYLINKConnection alloc] initWithConfig:config appKey:@"my app key"];
// All the messages in all the delegates are optional.
connection.lifeCycleDelegate = self;
connection.remotePeerDelegate = self;
connection.mediaDelegate = self;
connection.messagesDelegate = self;
connection.fileTransferDelegate = self;
```

### Connect to a room ###

There are two ways to connect to a room: 1) Using the secret, 2) Using the credentials.

**Using the secret:**

```
#!Objective-C

[connection connectToRoomWithSecret:@"my shared secret" roomName:@"my room name" userInfo:@"my display name"];
```

**Using the credentials:**

```
#!Objective-C

NSDictionary *credInfo = @{@"credential": @"credential", @"startTime":[NSDate date], @"duration":24}
[connection connectToRoomWithCredentials:credInfo roomName:@"my room name" userInfo:@"my display name"];
```
In this case, the credentials need to be calculated in a certain way. We are laying down our objective-c implementation below for reference. Please note that the startTime and duration used should be the same as that which are intended to pass while connecting to the room:

```
#!Objective-C

+ (NSString*)calculateCredentials:(NSString*)roomName duration:(CGFloat)duration startTime:(NSDate*)startTime secret:(NSString*)secret {
    NSLog(@"INFO::%s::room name=>%@", __PRETTY_FUNCTION__, roomName);
    NSLog(@"INFO::%s::call duration=>%f", __PRETTY_FUNCTION__, duration);
    NSLog(@"INFO::%s::start time=>%f", __PRETTY_FUNCTION__, duration);
    NSLog(@"DEBUG::%s::secret=>%@", __PRETTY_FUNCTION__, secret);
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneWithName:@"UTC"]];
    [dateFormatter setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss.0'Z'"];
    NSString *dateString = [dateFormatter stringFromDate:startTime];
    NSLog(@"INFO::%s::iso start time=>%@", __PRETTY_FUNCTION__, dateString);
    
    NSString *cred = [SKYLINKConnection hmac:[NSString stringWithFormat:@"%@_%f_%@", roomName, duration, dateString] withKey:secret];
    cred = CFBridgingRelease(CFURLCreateStringByAddingPercentEscapes(NULL, (__bridge CFStringRef)cred, NULL, (__bridge CFStringRef)@"!*'\"();:@&=+$,/?%#[]% ", CFStringConvertNSStringEncodingToEncoding(NSUTF8StringEncoding)));
    NSLog(@"INFO::%s::calculated credential=>%@", __PRETTY_FUNCTION__, cred);
    
    return cred;
}

// This function will calculate SHA hash and encode the output string as base 64
+ (NSString *)hmac:(NSString *)plaintext withKey:(NSString *)key {
    const char *cKey  = [key cStringUsingEncoding:NSASCIIStringEncoding];
    const char *cData = [plaintext cStringUsingEncoding:NSASCIIStringEncoding];
    
    uint8_t cHMAC[CC_SHA1_DIGEST_LENGTH];
    
    CCHmac(kCCHmacAlgSHA1, cKey, strlen(cKey), cData, strlen(cData), cHMAC);
    
    // Logging the hmac value ...
    NSString *Hash1 = @"";
    for (int i=0; i< CC_SHA1_DIGEST_LENGTH; i++)
        Hash1 = [Hash1 stringByAppendingString:[NSString stringWithFormat:@"%02X", cHMAC[i]]];
    NSLog(@"INFO::%s::Hash1->%@", __PRETTY_FUNCTION__, Hash1);
    
    // Base 64 encoding ...
    return [SKYLINKConnection base64forData:[[NSData alloc] initWithBytes:cHMAC length:sizeof(cHMAC)]];
}

+ (NSString*)base64forData:(NSData*)theData {
    const uint8_t* input = (const uint8_t*)[theData bytes];
    NSInteger length = [theData length];
    
    static char table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    
    NSMutableData* data = [NSMutableData dataWithLength:((length + 2) / 3) * 4];
    uint8_t* output = (uint8_t*)data.mutableBytes;
    
    NSInteger i;
    for (i=0; i < length; i += 3) {
        NSInteger value = 0;
        NSInteger j;
        for (j = i; j < (i + 3); j++) {
            value <<= 8;
            
            if (j < length) {  value |= (0xFF & input[j]);  }  }  NSInteger theIndex = (i / 3) * 4;  output[theIndex + 0] = table[(value >> 18) & 0x3F];
        output[theIndex + 1] = table[(value >> 12) & 0x3F];
        output[theIndex + 2] = (i + 1) < length ? table[(value >> 6) & 0x3F] : '=';
        output[theIndex + 3] = (i + 2) < length ? table[(value >> 0) & 0x3F] : '=';
    }
    
    return [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
}
```
# Connection Configuration: #

Configuration is represented by means of an instance of SKYLINKConnectionConfig which comprise of the following properties:

```
#!Objective-C

@property (nonatomic, unsafe_unretained) BOOL video;
@property (nonatomic, unsafe_unretained) BOOL audio;
@property (nonatomic, unsafe_unretained) BOOL fileTransfer;
@property (nonatomic, unsafe_unretained) BOOL dataChannel;
@property (nonatomic, unsafe_unretained) NSInteger timeout;
@property (nonatomic, strong) NSDictionary *userInfo;
```

It is self-explanatory but there are a few dependencies between some of the properties that are explained below:

### video & audio ###

____________________________________________________________
                         Input                         |                         Output                         
____________________________________________________________
           video           |           audio           |           video           |           audio           
____________________________________________________________
             T             |             T             |             T             |             T          
             T             |             F             |             F             |             F          
             F             |             T             |             F             |             T          
             F             |             F             |             F             |             F          
____________________________________________________________

### fileTransfer & dataChannel ###

____________________________________________________________
                         Input                         |                         Output                         
____________________________________________________________
        fileTransfer       |        dataChannel        |        fileTransfer       |        dataChannel        
____________________________________________________________
             T             |             T             |             T             |             T          
             T             |             F             |             F             |             F          
             F             |             T             |             F             |             T          
             F             |             F             |             F             |             F          
____________________________________________________________

**PS: If the call has video in it then the framework supports upto 4 external participants, if it has audio in it without video then the framework supports upto 8 external participants. With a call without an audio or video it supports upto 16 external participants**

# File Transfer: #

The current implementation of file transfer supports the transfer of one file at a time between two peers.

For sending a file one needs to send one of the following messages:

```
#!Objective-C

// This will raise an exception if there is already a file transfer being done with the same peer.
[connection sendFileTransferRequest:fileURL assetType:assetType peerId:@"id of the target peer"]; // For one to one transfer

// This will raise an exception if there is already a file being broadcast.
[connection sendFileTransferRequest:fileURL assetType:assetType]; // For one to all transfer
```

where assetType is represented by the following enum:

```
#!Objective-C

typedef enum SKYLINKAssetType {
    SKYLINKAssetTypeFile = 1, // Files within the app sandbox
    SKYLINKAssetTypeMusic, // Files from the music library
    SKYLINKAssetTypePhoto // Photo and Video content from the Photo Library
} SKYLINKAssetType;
```

The connection sends the following message to the fileTransferDelegate when it receives a file transfer request from a peer.

```
#!Objective-C

- (void)connection:(SKYLINKConnection*)connection didReceiveRequest:(NSString*)filename peerId:(NSString*)peerId;
```

One should accept or reject the file transfer request by sending the following the message:

```
#!Objective-C

[connection acceptFileTransfer:YES fileName:@"the same filename as received in the request" peerId:@"id of the peer sending the file"];
```

The connection sends the following message to the delegate to intimate it about the request acceptance or rejection. In case the request is accepted, it readily starts to send data to the other peer.

```
#!Objective-C

- (void)connection:(SKYLINKConnection*)connection didReceivePermission:(BOOL)isPermitted filename:(NSString*)filename peerId:(NSString*)peerId;
```

## Other fileTransferDelegate messages: ##

The connection sends the following message to the delegate to intimate any kind of halt state during file data transfer (timeout for example).

```
#!Objective-C

- (void)connection:(SKYLINKConnection*)connection didDropTransfer:(NSString*)filename reason:(NSString*)message isExplicit:(BOOL)isExplicit peerId:(NSString*)peerId;
```

There are two interpretations of the following message. If the connection is sending a file, it sends this message to the delegate to intimate file sending completion in which case fileData would be 'nil'. Whereas if the connection is receiving a file, it sends this message to the delegate to intimate file transfer completion.

```
#!Objective-C

- (void)connection:(SKYLINKConnection*)connection didCompleteTransfer:(NSString*)filename fileData:(NSData*)fileData peerId:(NSString*)peerId;
```

**Moreover for tracking the file transfer progress, the file transfer mechanism of the connection keeps on issuing an NSNotification with the name 'SKYLINKFileProgress' that has a float value against 'percentage' key inside the userInfo and a string against 'peerId' key to specify the remote Peer Id.**
